package com.example.crudwithvaadin;

import com.vaadin.flow.component.button.Button;
import com.vaadin.flow.component.icon.VaadinIcon;
import com.vaadin.flow.component.notification.Notification;
import com.vaadin.flow.component.orderedlayout.HorizontalLayout;
import com.vaadin.flow.component.orderedlayout.VerticalLayout;
import com.vaadin.flow.component.textfield.TextField;
import com.vaadin.flow.data.value.ValueChangeMode;
import org.vaadin.klaudeta.PaginatedGrid;
import org.springframework.util.StringUtils;

public class CustomerView extends VerticalLayout {

	private final CustomerRepository repo;

	private final CustomerEditor editor;

	final PaginatedGrid<Customer> grid;

	final TextField filter =  new TextField();
	final TextField grdSz =  new TextField();

	private final Button addNewBtn = new Button("New customer", VaadinIcon.PLUS.create());;
	HorizontalLayout actions = new HorizontalLayout(addNewBtn,filter, grdSz);

	public CustomerView(CustomerRepository repo, CustomerEditor editor) {
		this.repo = repo;
		this.editor = editor;
		this.grid = new PaginatedGrid<>();
//		this.setSizeFull();
		this.setSizeUndefined();
		grid.setPageSize(40);
		grid.setPaginatorSize(5);


		// build layout
		add(editor,actions, grid );

//		grid.setHeight("300px");
		grid.addColumn(Customer::getId).setHeader("ID").setSortable(true).setKey("id");
		grid.addColumn(Customer::getFirstName).setHeader("First Name").setSortable(true).setKey("firstName");
		grid.addColumn(Customer::getLastName).setHeader("Last Name").setSortable(true).setKey("lastName");
//		grid.setColumns("id", "firstName", "lastName");
		grid.getColumnByKey("id").setWidth("60px").setFlexGrow(0);

		filter.setPlaceholder("Filter by last name");
		grdSz.setPlaceholder("Grid Size: 10");

		// Hook logic to components

		// Replace listing with filtered content when user changes filter
		filter.setValueChangeMode(ValueChangeMode.EAGER);
		filter.addValueChangeListener(e -> listCustomers(e.getValue()));
		grdSz.addValueChangeListener(e -> setGridSize());

		// Connect selected Customer to editor or hide if none is selected
		grid.asSingleSelect().addValueChangeListener(e -> {
			editor.editCustomer(e.getValue());
		});

		// Instantiate and edit new Customer the new button is clicked
		addNewBtn.addClickListener(e -> editor.editCustomer(new Customer("", "")));

		// Listen changes made by the editor, refresh data from backend
		editor.setChangeHandler(() -> {
			editor.setVisible(false);
			listCustomers(filter.getValue());
		});

		// Initialize listing
		listCustomers(null);
	}

	// tag::listCustomers[]
	void listCustomers(String filterText) {
		if (StringUtils.isEmpty(filterText)) {
			grid.setItems(repo.findAll());
		}
		else {
			grid.setItems(repo.findByLastNameStartsWithIgnoreCase(filterText));
		}
	}
	// end::listCustomers[]
	void setGridSize(){
		String uiValue = grdSz.getValue();
		try {
			int convertedValue = Integer.parseInt(uiValue);
			grid.setPageSize(convertedValue);
			grid.refreshPaginator();
		} catch (NumberFormatException e) {
			 Notification.show("Could not convert value: " + uiValue);
			;
		}
	}


}
